module Main where

import TRS
import Rule (subset)
import Parser
import CompositionalCriteria
import System.Environment

main :: IO ()
main = do
  args <- getArgs
  case args of
    [k, smt, file] -> do
      spec <- readSpecFile file
      case spec of
        Left e -> print e
        Right trs -> process (read k) smt trs
    _ -> help

help :: IO ()
help = do
  putStrLn "reduce <k> <smt> <file.trs>"
  putStrLn ""
  putStrLn "where k is a depth for joinability and smt is a SMT solver."

process :: Int -> String -> TRS -> IO ()
process k smt trs = do
  trs' <- reduceRepeated k smt trs
  putStrLn (toCoCo trs trs')

reduceRepeated :: Int -> String -> TRS -> IO TRS
reduceRepeated k smt trs = do
  m <- reduceTRS smt k trs
  case m of
    Just trs' -> reduceRepeated k smt trs'
    Nothing -> return trs

toCoCo :: TRS -> TRS -> String
toCoCo orTRS redTRS = unlines [vars, rules, comment]
  where
    vars = "(VAR " ++ unwords (variables redTRS) ++ ")"
    rules = "(RULES\n" ++ showTRS redTRS ++ ")"
    comment = "(COMMENT\n" ++ txt ++ ")"
    txt = 
      if subset orTRS redTRS then
        "reduction failed"
      else
        "the following rules are removed from the original TRS\n"
        ++ showTRS [rl | rl <- orTRS, notElem rl redTRS]
